import 'dart:async';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;

class Category {
  Category(this.name) : devices = [];

  final String name;
  List<Device> devices;

  static Future<List<Category>> fetch(String base_url) async {
    final response = await http.get(Uri.parse("http://$base_url/devices"));

    if (response.statusCode != 200) {
      throw Exception("Failed to fetch devices");
    }

    final List<Category> categories = [];

    final Map<String, List<dynamic>> json =
        Map.castFrom(jsonDecode(jsonDecode(response.body)));

    for (MapEntry<String, List<dynamic>> category_entry in json.entries) {
      final Category category = Category(category_entry.key);

      for (dynamic device_info_dyn in category_entry.value) {
        final Map<String, dynamic> device_info = device_info_dyn;

        final String device_id = device_info['id'];
        final String? device_descriptor = device_info['desc'];
        final bool power_control = device_info['toggle'];

        final response =
            await http.get(Uri.parse("http://$base_url/plug_state/$device_id"));

        if (response.statusCode != 200) {
          throw Exception("Failed to fetch plug_state for $device_id");
        }

        final Map<String, dynamic> device_state =
            Map.castFrom(jsonDecode(jsonDecode(response.body)));

        final Device device = Device(
          device_id,
          device_descriptor,
          device_state["led"],
          device_state["power"],
          device_state["power_draw"],
          power_control && device_state["power_draw"] < 15,
        );

        category.devices.add(device);
      }

      categories.add(category);
    }

    return categories;
  }
}

class CategoryWidget extends StatelessWidget {
  Category category;

  CategoryWidget({super.key, required this.category});

  String capitalize(String s) {
    return "${s[0].toUpperCase()}${s.substring(1)}";
  }

  @override
  Widget build(BuildContext context) {
    var list = category.devices
        .map((device) => TableRow(children: [
              Text(
                  textAlign: TextAlign.center,
                  device.device_descriptor ?? device.device_id),
              TextButton(
                child: Text(
                    textAlign: TextAlign.center,
                    device.led_state ? 'On' : 'Off'),
                onPressed: () {},
              ),
              TextButton(
                child: Text(
                    textAlign: TextAlign.center,
                    device.power_state ? 'On' : 'Off'),
                onPressed: () {},
              ),
              Text(textAlign: TextAlign.center, "${device.power_draw} W")
            ]))
        .toList();

    list.insert(
        0,
        const TableRow(
            decoration: BoxDecoration(
              color: Colors.blueGrey,
            ),
            children: [
              Text(
                  style: TextStyle(fontWeight: FontWeight.bold),
                  textAlign: TextAlign.center,
                  "Name"),
              Text(
                  style: TextStyle(fontWeight: FontWeight.bold),
                  textAlign: TextAlign.center,
                  "LED"),
              Text(
                  style: TextStyle(fontWeight: FontWeight.bold),
                  textAlign: TextAlign.center,
                  "Power"),
              Text(
                  style: TextStyle(fontWeight: FontWeight.bold),
                  textAlign: TextAlign.center,
                  "Power Draw"),
            ]));

    return Column(children: <Widget>[
      Text(
          style: const TextStyle(fontWeight: FontWeight.bold),
          textScaleFactor: 2.0,
          capitalize(category.name)),
      Table(
          border: TableBorder.all(),
          defaultVerticalAlignment: TableCellVerticalAlignment.middle,
          children: list)
    ]);
  }
}

class Device {
  final String device_id;
  String? device_descriptor;
  bool led_state;
  bool power_state;
  double power_draw;
  bool power_control;

  Device(this.device_id, this.device_descriptor, this.led_state,
      this.power_state, this.power_draw, this.power_control);
}
